/***************************************************************************

Version History:

Ver.No	Comment							By
===========================================================================
1.0 	first version (seems to do things right)   		andy@mssx
1.1	some bugs fixed (tks to Maarten)			andy@mssx.uucp
								maart@cs.vu.nl
1.2 	ported to atari st - other enhancements - see below     randyh@hpsad

BUG REPORTS:
============
	- The offset count in the first column is wrong,
	  except for the first line; it's 0x10 too high.
								(fixed)
	- The test in disp() if a char is printable, fails
	  for chars >= 0177.
								(fixed)

	- Help message for 'H' incorrect			(fixed)


I declare this program as freeware, i.e. you may duplicate it, give it
to your friends, and transfer it to any machine you like, as long as
you do not change or delete the build in copyright message.

	Andreas Pleschutznig
	Teichhofweg 2
	8044 Graz
	Austria 

Comments and bug reports to:
	andy@mssx	(mcvax!tuvie!mssx!andy)


*****************************************************************************/

/*****************************************************************************
Version 1.2 notes: Atari ST port by Randy Hosler             Jan 13, 1992      

1) Removed curses and replaced with wind*() calls based on code from 
   Tim Thompson's original Stevie code.

2) Changed several command keys:
	'p' => 'b'
	'n' => 'f'
	'f' => '/'
	'n' now repeats previous search.

3) Added code to verify the loading of a new sector if the cuurent sector
   was edited and not yet written back to disk.

4) Changed edit_*() routines so that they don't redraw the whole screen on
   every value change.  For reasons of speed.

6) Also fixed edit_*() routines so that you cannot edit past EOF. Took cursor
   motion stuff out of edit_*() routines and put it in new move_curse().

7) Also sped up disp().

8) Fixed the 's' and 'f+' commands so that you cannot seek past the end of
   file.
*****************************************************************************/

#include <fcntl.h>
#include <stddef.h>
#include <stdio.h>
#include <osbind.h>

#define		CTRL(c)		((c) ^ 0100)
#define		DEL		CTRL('?')

#define		KEY_LEFT	CTRL('H')
#define		KEY_DOWN	CTRL('J')
#define		KEY_UP		CTRL('K')
#define		KEY_RIGHT	CTRL('L')
#define		O_RDWR		2

#define ASCX	63
#define ASCY	6
#define HEXY	6
#define HEXX	12

int 	ROWS, COLS;             /* max rows and cols */
int 	Row, Col;               /* current row and col */
int 	changed;                /* current sector edited */
int 	srch_ok;                /* is there a valid search string for '/' */
int 	searchpos;              /* current search position in sector */
char	srchstr[60];            /* string to search for */
int     path;                   /* path to file to patch */
long	filpos;			/* position in file */
long	max_size;		/* max size of file */
char	secbuf[256];		/* sector read buffer */

char	filename[60];           /* file we are editing */
int	length;			/* length of read sector */

main(argc,argv)
int argc;
char	**argv;
{
	if (argc != 2) {
		fprintf(stderr,"Usage: %s filename\n",argv[0]);
		exit(1);
	}
	if (( path = open(argv[1],O_RDWR)) == -1) {
		fprintf(stderr,"%s: Can't open '%s'\n",argv[0],argv[1]);
		exit(1);
	}
	max_size = fsize(argv[1]);
	sprintf(filename,"%s",argv[1]);
	windinit();
	filpos = 0;			/* set global position to 0 */
	length = 0;
	windclear();
	command();
	windclear();
	close(path);
	exit(0);
}

command()
{
	int inval;

	header("BPE Version 1.2",filename,"(C) 1988 MSS Graz");
	dump();
	inval = srch_ok = changed = 0;
	while ((inval != 'q') && (inval != 'Q')) {
		windgoto(2,0);
		clrtoeol();
		print_pos();
		windgoto(2,0);
		windstr("COMMAND : ");
		inval = windgetc();
		clrstat();
		switch (inval) {
			case 'q':
			case 'Q':
				if (changed) 
					if (dont_proceed())
						inval = ' ';
				break;
			case 'h':
			case 'H':
			case '?':
				help();
				break;
			case '/':
				if (changed) 
					if (dont_proceed())
						break;
				find_string(0);
				dump();
				break;
			case 'n':
			case 'N':
				if (srch_ok)  {
					if (changed) 
						if (dont_proceed())
							break;
					find_string(1);
					dump();
					}
				break;
			case '+':
			case 'f':
			case 'F':
				if (changed) 
					if (dont_proceed())
						break;
				if (filpos+256 <= max_size)  {
					searchpos = 0;
					filpos += 256;
					dump();
					}
				else
					wrtstat("At end of file !");
				break;
			case '-':
			case 'b':
			case 'B':
				if (changed) 
					if (dont_proceed())
						break;
				searchpos = 0;
				filpos -= 256;
				if (filpos < 0)
					filpos = 0;
				dump();
				break;
			case 'D':
			case 'd':
				if (changed) 
					if (dont_proceed())
						break;
				dump();
				break;
			case 's':
			case 'S':
				if (changed) 
					if (dont_proceed())
						break;
				set();
				searchpos = 0;
				if (filpos >= max_size)  {
					filpos = max_size - 256;
					if (filpos < 0)
						filpos = 0;
					wrtstat("At end of file !");
					}
				dump();
				break;
			case 'e':
				edit_ascii();
				break;
			case 'E':
				edit_hex();
				break;
			case 'w':
			case 'W':
				wrsec();
				changed = 0;
				break;
			default:
				wrtstat("Invalid Command !");
		}
	}
}

move_curse(val, curx, cury)
int val, *curx, *cury;
{
	switch (val) {
		case KEY_UP:
			if (*cury)
				(*cury)--;
			else
				beep();
			break;
		case KEY_DOWN:
			if (*cury*16 + 16 + *curx < length)
				(*cury)++;
			else
				beep();
			break;
		case KEY_RIGHT:
			if (*curx < (length - *cury*16 - 1))  {
				(*curx)++;
				if (*curx == 16)  {
					*curx = 0;
					if (*cury*16 + 16 + *curx < length)
						(*cury)++;
					}
				}
			else
				beep();
			break;
		case KEY_LEFT:
			if (*curx)
				(*curx)--;
			else
				beep();
			break;
		}
}


edit_ascii()
{
	int inval = 0;
	int cury,curx;
	char s[8];

	if (length == 0)
		length = dump();
	windgoto(2,0);
	clrtoeol();
	windstr("Left ^H - down ^J - up ^K - right ^L - end editing with ^E");
	curx = cury = 0;
	while (inval != CTRL('E')) {
		windgoto(ASCY+cury,ASCX+curx);
		inval = windgetc();
		if ((inval >= 0x20) && (inval <= 0x7e)) {
			changed = 1;
			secbuf[cury*16+curx] = inval;
			windgoto(ASCY+cury,ASCX+curx);
			windputc(inval);
			windgoto(ASCY+cury,HEXX+curx*3);
			sprintf(s,"%02X",inval & 0xFF);
			windstr(s);
			inval = KEY_RIGHT;
			}
		move_curse(inval, &curx, &cury);
		}
}

gethex(cury,curx,inlen)
int	cury,curx,*inlen;
{
	int val;
	int hexval;
	int value;
	char s[8];

	*inlen = 0;
	while (*inlen < 2) {
		val = windgetc();
		if (val > 255 && *inlen == 0) 
			return(val);
		if (val < '0' && *inlen == 0)
			return(val);
		if (val > '9') val &= ~0x20;
		if (((val >= '0') && (val <='9')) || 
		    ((val >= 'A') && (val <= 'F'))) {
			if (val <= '9')
				hexval = val - '0';
			else
				hexval = val - 'A' + 0x0a;
			switch (*inlen) {
				case 0:
					value = hexval << 4;
					windputc(val);
					break;
				case 1:
					value += hexval;
					windputc(val);
					break;
				}
			(*inlen)++;
			}
		else    {
			beep();
			sprintf(s,"%02X",secbuf[cury*16+curx] & 0xFF);
			windgoto(HEXY+cury,HEXX+curx*3+*inlen);
			windputc(s[*inlen]);
			windgoto(HEXY+cury,HEXX+curx*3+*inlen);
			}
		}
	return(value);
}

edit_hex()
{
	int inlen;
	int inval = 0;
	int cury,curx;

	if (length == 0)
		length = dump();
	windgoto(2,0);
	clrtoeol();
	windstr("Left ^H - down ^J - up ^K - right ^L - end editing with ^E");
	curx = cury = 0;
	while (inval != CTRL('E')) {
		windgoto(HEXY+cury,HEXX+curx*3);
		inval = gethex(cury,curx,&inlen);
		if (inlen == 0)
			move_curse(inval, &curx, &cury);
		else  {
			if ((inval >= 0x00) && (inval <= 0xff)) {
				changed = 1;
				secbuf[cury*16+curx] =inval;
				windgoto(ASCY+cury,ASCX+curx);
				if (' ' <= inval && inval < DEL)
					windputc(inval);
				else
					windputc('.');
				move_curse(KEY_RIGHT, &curx, &cury);
			}
		}
	}
}

find_string(next)
int next;
{
	int	found;
	int 	stlen;
	char 	s[80];

	if (next != 1)  {
		windgoto(2,0);
		clrtoeol();
		windstr("String to search : ");
		gets(srchstr);
		searchpos = 0;
		}
	windgoto(2,0);
	clrtoeol();
	sprintf(s,"Searching for '%s'",srchstr);
	windstr(s);
	found = 0;
	stlen = strlen(srchstr);
	while (found == 0) {
		while ((256 - searchpos) >= stlen) {
			if (testchar(secbuf+searchpos,srchstr,stlen))
				searchpos++;
			else {
				filpos += searchpos;
				searchpos = found = srch_ok = 1;
				break;
			}
		}
		if (found == 0) {
			filpos += searchpos;
			searchpos = 0;
		}
		if (rdsec() == 0) {
			srch_ok = 0;  /* turn off 'search next' if not found */
			found = 1;    /* not really, just want out of loop   */	
		}
	}
}

testchar(buffer,string,length)
char	*buffer;
char	*string;
int	length;
{
	register int i;
	
	i = 0;
	while ( i < length) {
		if (buffer[i] != string[i])
			break;
		i++;
	}
	if ( i == length)
		return(0);
	return(1);
}

set()
{
	windgoto(2,0);
	clrtoeol();
	windstr("New File Position : ");
	scanf("%lx",&filpos);
}

disp(length)
int	length;
{
	int	i, j, c;
	char	s[80], *sptr;

	j = 0;

	windgoto(4,0);
	windstr( " ADDRESS    00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F      ASCII");

	windgoto(5,0);
	windstr("===============================================================================");

	for ( i = 0; i < 16; i++) {
		windgoto(ASCY+i,0);
		sprintf(s,"%08lX",filpos+i*16);
		windstr(s);
		sptr = s;
		*sptr = '\0';
		for (j = 0; j < 16; j++) {
			if (( i*16 + j ) >= length)
				goto Disp1;
			sprintf(sptr,"%02X ",secbuf[i*16+j] & 0xFF);
			sptr += 3;
		}
Disp1:
		windgoto(ASCY+i,HEXX);
		windstr(s);
		clrtoeol();
		for (j = 0; j < 16; j++) {
			if (( i*16 + j ) >= length) {
				s[j] = '\0';
				windgoto(ASCY+i,ASCX);
				windstr(s);
				clrtobot();
				goto Disp2;
			}
			if (' ' <= (c = secbuf[i * 16 + j]) && c < DEL)
				s[j] = c;
			else
				s[j] = '.';
		}
		s[j] = '\0';
		windgoto(ASCY+i,ASCX);
		windstr(s);
	}
Disp2:
	;
}

dump()
{
	if (length = rdsec())
		disp(length);
	return(length);
}

rdsec()
{
	lseek(path,filpos,0);
	length = read(path,secbuf,256);
	return(length);
}

wrsec()
{
	lseek(path,filpos,0);
	write(path,secbuf,length);
	wrtstat("Sector written.");
}

print_pos()
{
	char	s[80];

	windgoto(2,55);
	sprintf(s,"Rel. Position : %08lX",filpos);
	windstr(s);
}

help()
{
	windclear();
	header("BPE v1.2","Binary Program Editor for Atari ST","1992 R Hosler");
	windgoto(3,10);
	windstr("Valid Commands are :");
	windgoto(5,15);
	windstr("D - Dump one page from current file position");
	windgoto(6,15);
	windstr("S - Set current file pointer");
	windgoto(7,15);
	windstr("/ - Find string in file (beginning from curr. position)");
	windgoto(8,15);
	windstr("N - Find next occurrence of string");
	windgoto(9,15);
	windstr("+ - Display next sector forward");
	windgoto(10,15);
	windstr("F - Display next sector forward");
	windgoto(11,15);
	windstr("- - Display previous sector back");
	windgoto(12,15);
	windstr("B - Display previous sector back");
	windgoto(13,15);
	windstr("e - Edit ASCII portion of file");
	windgoto(14,15);
	windstr("E - Edit binary portion of file");
	windgoto(15,15);
	windstr("W - Write modified sector back to disk");
	windgoto(18,15);
	windstr("Q - Quit Program");
	wrtstat("Continue with any char.");
	windgetc();
	windclear();
	header("BPE Version 1.2",filename,"(C) 1988 MSS Graz");
	disp(length);
}

wrtstat(statstr)
char    *statstr;
{
	beep();
	windgoto(ROWS-1,40-strlen(statstr)/2);
	windstr(statstr);
}

clrstat()
{
	windgoto(ROWS-1,0);
	clrtoeol();
}

dont_proceed()
{
	int in, c;

	wrtstat("Current sector changed but not written !");
	windgoto(2,0);
	clrtoeol();
	windstr("[W]rite and proceed, [P]roceed anyway or [C]ancel ? ");
	c = Col;
	do {
		in = windgetc();
		windgoto(2,c);
		in += 'A' - 'a';
	} while (in != 'P' && in != 'C' && in != 'W');
	clrstat();
	if (in == 'C')
		return(1);
	if (in == 'W')
		wrsec();
	changed = 0;
	return(0);
}

header(left,mid,right)
char    *left;
char    *mid;
char    *right;
{
	windgoto(0,0);
	windstr(left);
	windgoto(0,79-strlen(right));
	windstr(right);
	windgoto(0,40-strlen(mid)/2);
	windstr(mid);
}

/* ******************************************** */
/* ******************************************** */
/* from here down is the machine dependent i/o  */
/* ******************************************** */
/* ******************************************** */

windinit()
{
	COLS=80;
	ROWS=25;
}

windgoto(r,c)
int r,c;
{
	Cconout('\033');
	Cconout('Y');
	Cconout(r+040);
	Cconout(c+040);
	Row = r;
	Col = c;
}

windclear()
{
	Cconws("\033H\033J");
}

windgetc()
{
	return(Cnecin());
}

windstr(s)
char *s;
{
	Cconws(s);
	Col = Col + strlen(s);
	if (Col == COLS)  {
		Col = 0;
		Row++;
		if (Row == ROWS)
			Row--;
	}
}

windputc(c)
int c;
{
	Cconout(c);
	Col++;
	if (Col == COLS)  {
		Col = 0;
		Row++;
		if (Row == ROWS)
			Row--;
	}
}

beep()
{
	Cconout('\007');
}

clrtoeol()
{
	char buf[80], *ptr;
	int  i;
	int  rr, cc;

	rr = Row;
	cc = Col;
        ptr = buf;

	for (i = Col; i < 80; i++)
		*ptr++ = ' ';
	*ptr = '\0';
	windgoto(Row, Col);
	windstr(buf);
	windgoto(rr, cc);
}

clrtobot()
{
	int r, rr, cc;

	rr = Row;
	cc = Col;
	r = Row + 1;
	
	while (r < ROWS)   {
		windgoto(r++, 0);
		clrtoeol();
	}
	windgoto(rr, cc);
}
